import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:oauth2/oauth2.dart' as oauth2;

class LoginPage extends StatelessWidget {
  const LoginPage({super.key});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Goatworks Login'),
      ),
      body: Center(
        child: ElevatedButton(
          onPressed: () async {
            await _login(context);
          },
          child: const Text('Login with Goatpass'),
        ),
      ),
    );
  }

  Future<void> _login(BuildContext context) async {
    final authorizationEndpoint =
        Uri.parse('https://id.smartsheep.studio/auth/o/connect');
    final tokenEndpoint =
        Uri.parse('https://id.smartsheep.studio/api/auth/token');
    final userinfoEndpoint =
        Uri.parse('https://id.smartsheep.studio/api/users/me');
    final redirectUrl = Uri.parse('goatagent://auth');

    const clientId = "goatagent";
    const clientSecret = "_F4%q2Eea3";

    const storage = FlutterSecureStorage();
    const storageKey = "identity";

    Future<oauth2.Client> createClient() async {
      // If logged in
      if (await storage.containsKey(key: storageKey)) {
        var credentials = oauth2.Credentials.fromJson(
            storage.read(key: storageKey) as String);
        return oauth2.Client(credentials,
            identifier: clientId, secret: clientSecret);
      }

      var grant = oauth2.AuthorizationCodeGrant(
        clientId,
        authorizationEndpoint,
        tokenEndpoint,
        secret: clientSecret,
        basicAuth: false,
      );

      var authorizationUrl = grant.getAuthorizationUrl(redirectUrl);

      if (Platform.isAndroid || Platform.isIOS) {
        // Let Goatpass know it is embed in an app
        authorizationUrl.replace(
            queryParameters: {"embedded": "yes"}
              ..addAll(authorizationUrl.queryParameters));

        // Use WebView to get authorization url
        var responseUrl = await Navigator.of(context).push(MaterialPageRoute(
          builder: (context) => AuthorizationPage(authorizationUrl),
        ));

        var responseUri = Uri.parse(responseUrl);
        return await grant
            .handleAuthorizationResponse(responseUri.queryParameters);
      } else {
        // TODO Use system browser to get url
        throw UnimplementedError("unsupported platform");
      }
    }

    try {
      var client = await createClient();
      var response = await client.read(userinfoEndpoint);

      print('Logged in: $response');
    } catch (e) {
      print(e);
    }
  }
}

class AuthorizationPage extends StatelessWidget {
  final Uri authorizationUrl;

  const AuthorizationPage(this.authorizationUrl, {super.key});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Connect with Goatpass'),
      ),
      body: WebViewWidget(
        controller: WebViewController()
          ..setJavaScriptMode(JavaScriptMode.unrestricted)
          ..setBackgroundColor(Colors.indigo)
          ..setNavigationDelegate(NavigationDelegate(
            onNavigationRequest: (NavigationRequest request) {
              if (request.url.startsWith('goatagent://auth')) {
                Navigator.of(context).pop(request.url);
                return NavigationDecision.prevent;
              }
              return NavigationDecision.navigate;
            },
          ))
          ..loadRequest(authorizationUrl),
      ),
    );
  }
}
